%define anolis_release 1
%define libname libdwarves
%define libver 1

Name: dwarves
Version: 1.27
Release: %{anolis_release}%{?dist}
License: GPLv2
Summary: Debugging Information Manipulation Tools (pahole & friends)
URL: https://github.com/acmel/dwarves
Source0: https://fedorapeople.org/~acme/dwarves/%{name}-%{version}.tar.xz
# core: Initialize cu->node with INIT_LIST_HEAD()
Patch: https://github.com/acmel/dwarves/commit/6a2b27c0f512619b0e7a769a18a0fb05bb3789a5.patch
# dwarf_loader: Add missing cus__add(cus, cu) to cus__merge_and_process_cu()
Patch: https://github.com/acmel/dwarves/commit/94a01bde592c555b3eb526aeb4c2ad695c5660d8.patch
Requires: %{libname} = %{version}-%{release}
BuildRequires: gcc
BuildRequires: cmake >= 2.8.12
BuildRequires: zlib-devel
BuildRequires: elfutils-devel >= 0.130
Provides: pahole

%description
dwarves is a set of tools that use the debugging information inserted in
ELF binaries by compilers such as GCC, used by well known debuggers such as
GDB, and more recent ones such as systemtap.

Utilities in the dwarves suite include pahole, that can be used to find
alignment holes in structs and classes in languages such as C, C++, but not
limited to these.

It also extracts other information such as CPU cacheline alignment, helping
pack those structures to achieve more cache hits.

These tools can also be used to encode and read the BTF type information format
used with the Linux kernel bpf syscall, using 'pahole -J' and 'pahole -F btf'.

A diff like tool, codiff can be used to compare the effects changes in source
code generate on the resulting binaries.

Another tool is pfunct, that can be used to find all sorts of information about
functions, inlines, decisions made by the compiler about inlining, etc.

One example of pfunct usage is in the fullcircle tool, a shell that drivers
pfunct to generate compileable code out of a .o file and then build it using
gcc, with the same compiler flags, and then use codiff to make sure the
original .o file and the new one generated from debug info produces the same
debug info.

Pahole also can be used to use all this type information to pretty print raw data
according to command line directions.

Headers can have its data format described from debugging info and offsets from
it can be used to further format a number of records.

The btfdiff utility compares the output of pahole from BTF and DWARF to make
sure they produce the same results.

%package -n %{libname}
Summary: Debugging information  processing library
Provides: %{libname}%{libver} = %{version}-%{release}

%description -n %{libname}
Debugging information processing library.

%package -n %{libname}-devel
Summary: Debugging information library development files
Requires: %{libname} = %{version}-%{release}
Provides: %{libname}%{libver}-devel = %{version}-%{release}

%description -n %{libname}-devel
Debugging information processing library development files.

%package doc
Summary: Doc files for dwarves
Requires: %{name} = %{version}-%{release}
BuildArch: noarch

%description doc
Doc files for dwarves

%prep
%autosetup -n %{name}-%{version} -p1

%build
%cmake -DCMAKE_BUILD_TYPE=Release .
%cmake_build

%install
%cmake_install

%ldconfig_scriptlets -n %{libname}

%generate_compatibility_deps

%files
%{_bindir}/btfdiff
%{_bindir}/codiff
%{_bindir}/ctracer
%{_bindir}/dtagnames
%{_bindir}/fullcircle
%{_bindir}/pahole
%{_bindir}/pdwtags
%{_bindir}/pfunct
%{_bindir}/pglobal
%{_bindir}/prefcnt
%{_bindir}/scncopy
%{_bindir}/syscse
%{_bindir}/ostra-cg
%dir %{_datadir}/dwarves/
%dir %{_datadir}/dwarves/runtime/
%dir %{_datadir}/dwarves/runtime/python/
%defattr(0644,root,root,0755)
%{_mandir}/man1/pahole.1*
%{_datadir}/dwarves/runtime/Makefile
%{_datadir}/dwarves/runtime/linux.blacklist.cu
%{_datadir}/dwarves/runtime/ctracer_relay.c
%{_datadir}/dwarves/runtime/ctracer_relay.h
%attr(0755,root,root) %{_datadir}/dwarves/runtime/python/ostra.py*
%{abidir}/btfdiff-option.list
%{abidir}/codiff-option.list
%{abidir}/ctracer-option.list
%{abidir}/fullcircle-option.list 
%{abidir}/pahole-option.list
%{abidir}/pdwtags-option.list
%{abidir}/pfunct-option.list
%{abidir}/pglobal-option.list
%{abidir}/scncopy-option.list
%{abidir}/syscse-option.list

%files -n %{libname}
%dir %{abidir}
%{_libdir}/%{libname}.so.*
%{_libdir}/%{libname}_emit.so.*
%{_libdir}/%{libname}_reorganize.so.*
%{abidir}/libdwarves.dump
%{abidir}/libdwarves_emit.dump
%{abidir}/libdwarves_reorganize.dump

%files -n %{libname}-devel
%doc MANIFEST README
%{_includedir}/dwarves/btf_encoder.h
%{_includedir}/dwarves/config.h
%{_includedir}/dwarves/ctf.h
%{_includedir}/dwarves/dutil.h
%{_includedir}/dwarves/dwarves.h
%{_includedir}/dwarves/dwarves_emit.h
%{_includedir}/dwarves/dwarves_reorganize.h
%{_includedir}/dwarves/elfcreator.h
%{_includedir}/dwarves/elf_symtab.h
%{_includedir}/dwarves/gobuffer.h
%{_includedir}/dwarves/hash.h
%{_includedir}/dwarves/libctf.h
%{_includedir}/dwarves/list.h
%{_includedir}/dwarves/rbtree.h
%{_libdir}/%{libname}.so
%{_libdir}/%{libname}_emit.so
%{_libdir}/%{libname}_reorganize.so

%files doc
%doc README.ctracer
%doc README.btf
%doc changes-v1.27
%doc NEWS

%changelog
* Mon Mar 17 2025 wangzhe <wanglan.wz@alibaba-inc.com> - 1.27-1
- update to 1.27
- Add 'Provides: pahole' to allow the package to be installed using $dnf install pahole
- Add patches to fix clang builds bug
  [Patches cherry-pick from c10s. Original changelog:
   Backport upstream bugfixes for clang builds]

* Thu Apr 11 2024 mgb01105731 <mgb01105731@alibaba-inc.com> - 1.26-1
- update to 1.26

* Thu Feb 16 2023 happy_orange <songnannan@linux.alibaba.com> - 1.24-2
- rebuild 

* Tue Feb 14 2023 happy_orange <songnannan@linux.alibaba.com> - 1.24-1
- update to 1.24

* Sat Oct 08 2022 Chunmei Xu <xuchunmei@linux.alibaba.com> - 1.23-1
- downgrade to 1.23
- fix kernel build failed with "FAILED: load BTF from vmlinux: Invalid argument"

* Mon Sep 19 2022 Chunmei Xu <xuchunmei@linux.alibaba.com> - 1.24-1
- init from upstream
